import { useEffect, useState } from "react";
import { getSnapshot } from "mobx-state-tree";
import { reaction } from "mobx";
function isMSTNode(value) {
    return Boolean(value && typeof value === "object" && "toJSON" in value);
}
export function useObservable(model) {
    const [state, setState] = useState({});
    useEffect(() => {
        if (!model)
            return;
        const disposers = [];
        // Set up reaction for each property
        Object.keys(state).forEach(prop => {
            const disposer = reaction(() => {
                const value = model[prop];
                if (isMSTNode(value)) {
                    // Handle nested properties
                    const nested = {};
                    Object.keys(state)
                        .filter(key => key.startsWith(`${prop}.`))
                        .forEach(key => {
                        const nestedProp = key.split(".")[1];
                        if (nestedProp) {
                            nested[nestedProp] = value[nestedProp];
                        }
                    });
                    return { value, ...nested };
                }
                return value;
            }, newValue => {
                setState(prev => ({
                    ...prev,
                    // @ts-expect-error - need to figure out some of this weird typing
                    [prop]: isMSTNode(newValue) ? newValue.value : newValue
                }));
            }, { fireImmediately: true });
            disposers.push(disposer);
        });
        return () => disposers.forEach(d => d());
    }, [model, Object.keys(state).join(",")]);
    return new Proxy({}, {
        get: (_, property) => {
            // Initialize tracking for new properties
            if (!(property in state)) {
                const value = model[property];
                setState(prev => ({
                    ...prev,
                    [property]: value
                }));
                if (typeof value === "function") {
                    return value.bind(model);
                }
                if (isMSTNode(value)) {
                    return new Proxy({}, {
                        get: (_, nestedProp) => {
                            const nestedValue = value[nestedProp];
                            if (typeof nestedValue === "function") {
                                return nestedValue.bind(value);
                            }
                            // Track nested property
                            const fullProp = `${String(property)}.${String(nestedProp)}`;
                            if (!(fullProp in state)) {
                                setState(prev => ({
                                    ...prev,
                                    [fullProp]: nestedValue
                                }));
                            }
                            return nestedValue;
                        }
                    });
                }
                return value;
            }
            const value = model[property];
            if (typeof value === "function") {
                return value.bind(model);
            }
            if (isMSTNode(value)) {
                return new Proxy({}, {
                    get: (_, nestedProp) => {
                        const nestedValue = value[nestedProp];
                        if (typeof nestedValue === "function") {
                            return nestedValue.bind(value);
                        }
                        const fullProp = `${String(property)}.${String(nestedProp)}`;
                        return state[fullProp] ?? nestedValue;
                    }
                });
            }
            return state[property] ?? value;
        }
    });
}
//# sourceMappingURL=useObservable.js.map